/*
 Alarm System Monitor - Server
 Version: 2.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 15/08/2012
 Last Modified: 27/12/2013
 
 The server listens for incoming requests from the sensors and tells them whether they should switch themselves on or off.
 If a sensor sends the alarm triggered command, we give priority to the sirens and remote controls and ignore the other sensors.
 
 Turning the alarm system on or off is done by remote control (WIP).
 
 */
 
// ATtiny24/44/84 Pin map
//
//               +-\/-+
//         VCC  1|o   |14  GND
//         PB0  2|    |13  PA0 
//         PB1  3|    |12  PA1 
//   Reset PB3  4|    |11  PA2 Trigger SMS Sender
//      CE PB2  5|    |10  PA3
//     CSN PA7  6|    |9   PA4 SCK
//      MI PA6  7|    |8   PA5 MO
//               +----+
//

#define F_CPU 1000000 // 1 MHz

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <string.h>
#include "hmac-sha1/hmac-sha1.h"
#include "setup.c"
#include "mirf.c"

volatile uint8_t system_state = SYSTEM_ON; // Current system state for the device, on or off
volatile uint8_t siren_state = SIREN_OFF; // Is the alarm's siren on or off
volatile uint8_t sixteen_second_timeout = false;

// HMAC 256bit random key (you should change this)
uint8_t hmac_key[] = {0x01, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
							0x88, 0x99, 0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF,
							0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF, 
							0x02, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77};

// 128bit rolling code for sensors (you should change this)
uint8_t sensor_rolling_code[] = {0x25, 0xF9, 0x34, 0x5A, 0xE2, 0xFD, 0x8D, 0x00,
											0x4C, 0x9A, 0xF7, 0x7F, 0x69, 0xD1, 0x3F, 0x82,};
uint16_t sensor_rolling_code_addition = 0;

// 128bit rolling code for remote control (you should change this)
uint8_t remote_control_rolling_code[] = {0xD8, 0xBF, 0xE9, 0x55, 0x9E, 0x32, 0xA7, 0x69, 
													  0x01, 0xEB, 0x8B, 0x62, 0xB7, 0x50, 0x41, 0x9F};
uint32_t remote_control_rolling_code_addition = 0;

uint8_t data_in[24];
uint8_t data_out[24];

int main(void) {
	setup();
	mirf_init();
	_delay_ms(50);
	mirf_config();
	
	while(1) {
		RX_POWERUP;
		mirf_flush_rx_tx(); // Flush RX and TX FIFO
		mirf_CE_hi; // Start listening
		
		// Wait for incoming requests
		while (!(mirf_status() & (1<<RX_DR))) { 
			_delay_us(250);
		}
		mirf_CE_lo; // Stop listening
		
		// Process the request
		if (!process_check_in()) {
			for (int x = 0; x < 5; x++) {
				mirf_CSN_lo;
				_delay_ms(50);
				mirf_CSN_hi;
				_delay_ms(50);
			}
		}
		else {
			mirf_CSN_lo;
			_delay_ms(50);
			mirf_CSN_hi;
			_delay_ms(50);
		}
	}
}

ISR(WDT_vect) { }
